// general dependencies
var express = require('express');
var path = require('path');
var favicon = require('serve-favicon');
var logger = require('morgan');
var cookieParser = require('cookie-parser');
var bodyParser = require('body-parser');

// server config dependencies
var endpoints = require('./include/endpoints').END_POINTS;
var pages = require('./include/pages').PAGES;
var configFile = require('./include/commons').CONFIG_FILE;
var config = require('./resources/config/' + configFile);

// routes dependencies
var index = require('./controllers/index');

var users = require('./endpoints/users');
var messages = require('./endpoints/messages');
var management = require('./endpoints/management');

console.log('==============================================');
console.log('');
console.log(new Date().toLocaleString() + ': Starting IM server, version ' + config.version);
console.log('');
console.log('==============================================');

// Application entry point
var app = express();

// Setup view engine as Jade
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'jade');

app.use(favicon(__dirname + '/public/favicon.ico', null));
app.use(logger('dev'));
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: false }));
app.use(cookieParser());
app.use(express.static(path.join(__dirname, 'public')));

app.use(pages.Home.Index, index);

app.use(endpoints.Msg.Base, messages);
app.use(endpoints.Users.Base, users);
app.use(endpoints.Management.Base, management);

// 捕捉404错误，并转发至错误处理器
app.use(function(req, res, next) {
  var err = new Error('Not Found');
  err.status = 404;
  next(err);
});

// In development mode, error handler will print stacktrace
if (app.get('env') === 'development') {
  app.use(function(err, req, res, next) {
    res.status(err.status || 500);
    res.render('error', {
      message: err.message,
      error: err
    });
  });
}

// In production mode, error handler no stack traces leaked to users
app.use(function(err, req, res, next) {
  res.status(err.status || 500);
  res.render('error', {
    message: err.message,
    error: {}
  });
});

process.on('uncaughtException', function (err) {
  console.log(err);
  console.log(err.stack);
});

module.exports = app;
